@@ -1,6 +1,8 @@
 #!/usr/bin/env perl
 # Original: https://github.com/defunkt/mustache/blob/master/bin/mustache
 
+# PODNAME: mustache.pl
+
 use strict;
 use warnings;
 use autodie;
@@ -30,6 +32,20 @@ else {
     }
 }
 
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+mustache.pl
+
+=head1 VERSION
+
+version 0.5.6
+
 =head1 NAME
 
 mustache.pl - Command line frontend for mustache logic-less templates.
@@ -46,4 +62,29 @@ Examples:
   $ cat data.yml | mustache.pl - template.mustache
   $ cat data.yml template.mustache | mustache.pl
 
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Pieter van de Bruggen <pvande@cpan.org>
+
+=item *
+
+Yanick Champoux <yanick@cpan.org>
+
+=item *
+
+Ricardo Signes <rjbs@cpan.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Pieter van de Bruggen.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
