@@ -1,64 +1,17 @@
 package Template::Mustache;
+our $AUTHORITY = 'cpan:YANICK';
+# ABSTRACT: Drawing Mustaches on Perl for fun and profit
+$Template::Mustache::VERSION = '0.5.6';
 use strict;
 use warnings;
 
-=head1 NAME
-
-Template::Mustache
-
-=head1 SYNOPSIS
-
-    use Template::Mustache;
-
-    print Template::Mustache->render(
-        "Hello {{planet}}", {planet => "World!"}), "\n";
-
-=head1 DESCRIPTION
-
-Template::Mustache is an implementation of the fabulous Mustache templating
-language for Perl 5.8 and later.
-
-See L<http://mustache.github.com>.
-
-=head1 AUTHOR
-
-Pieter van de Bruggen
-
-=cut
 
 use HTML::Entities;
 use File::Spec;
 use Scalar::Util 'blessed';
 
-use version 0.77; our $VERSION = qv('v0.5.5');
-
 my %TemplateCache;
 
-=head2 Functions
-
-=over 4
-
-=item build_pattern($otag, $ctag)
-
-Constructs a new regular expression, to be used in the parsing of Mustache
-templates.
-
-=over 4
-
-=item $otag
-
-The tag opening delimiter.
-
-=item $ctag
-
-The tag closing delimiter.
-
-=back
-
-Returns a regular expression that will match tags with the specified
-delimiters.
-
-=cut
 
 sub build_pattern {
     my ($otag, $ctag) = @_;
@@ -75,22 +28,6 @@ sub build_pattern {
     /xsm;
 }
 
-=item read_file($filename)
-
-Reads a file into a string, returning the empty string if the file does not
-exist.
-
-=over 4
-
-=item $filename
-
-The name of the file to read.
-
-=back
-
-Returns the contents of the given filename, or the empty string.
-
-=cut
 
 sub read_file {
     my ($filename) = @_;
@@ -103,75 +40,6 @@ sub read_file {
     return $data;
 }
 
-=item parse($tmpl, [$delims, [$section, $start]])
-
-Can be called in one of three forms:
-
-=over 4
-
-=item parse($tmpl)
-
-Creates an AST from the given template.
-
-=over 4
-
-=item $tmpl
-
-The template to parse.
-
-=back
-
-An array reference to the AST represented by the given template.
-
-=item parse($tmpl, $delims)
-
-Creates an AST from the given template, with non-standard delimiters.
-
-=over 4
-
-=item $tmpl
-
-The template to parse.
-
-=item $delims
-
-An array reference to the delimiter pair with which to begin parsing.
-
-=back
-
-Returns an array reference to the AST represented by the given template.
-
-=item parse($tmpl, $delims, $section, $start)
-
-Parses out a section tag from the given template.
-
-=over 4
-
-=item $tmpl
-
-The template to parse.
-
-=item $delims
-
-An array reference to the delimiter pair with which to begin parsing.
-
-=item $section
-
-The name of the section we're parsing.
-
-=item $start
-
-The index of the first character of the section.
-
-=back
-
-Returns an array reference to the raw text of the section (first element),
-and the index of the character immediately following the close section tag
-(last element).
-
-=back
-
-=cut
 
 sub parse {
     my ($tmpl, $delims, $section, $start) = @_;
@@ -276,34 +144,9 @@ sub parse {
     return \@buffer;
 }
 
-=item generate($parse_tree, $partials, @context)
-
-Produces an expanded version of the template represented by the given parse
-tree.
-
-=over 4
-
-=item $parse_tree
-
-The AST of a Mustache template.
-
-=item $partials
-
-A subroutine that looks up partials by name.
-
-=item @context
-
-The context stack to perform key lookups against.
-
-=back
-
-Returns the fully rendered template as a string.
-
-=cut
 
 sub generate {
     my ($parse_tree, $partials, @context) = @_;
-
     # Build a helper function to abstract away subtemplate expansion.
     # Recursively calls generate after parsing the given template.  This allows
     # us to use the call stack as our context stack.
@@ -378,27 +221,6 @@ sub generate {
     } @$parse_tree;
 }
 
-=item lookup($field, @context)
-
-Performs a lookup of a C<$field> in a context stack.
-
-=over 4
-
-=item $field
-
-The field to look up.
-
-=item @context
-
-The context stack.
-
-=back
-
-Returns the context element and value for the given C<$field>.
-
-=back
-
-=cut
 
 sub lookup {
     my ($field, @context) = @_;
@@ -407,10 +229,28 @@ sub lookup {
     for my $index (0..$#{[@context]}) {
         $ctx = $context[$index];
         my $blessed_or_not_ref = blessed($ctx) || !ref $ctx;
-        if (ref $ctx eq 'HASH') {
-            next unless exists $ctx->{$field};
+
+        if($field =~ /\./) {
+            # Dotted syntax foo.bar
+            my ($var, $field) = $field =~ /(.+?)\.(.+)/;
+
+            if(ref $ctx eq 'HASH') {
+                next unless exists $ctx->{$var};
+                ($ctx, $value) = lookup($field, $ctx->{$var});
+                last;
+            } elsif(ref $ctx eq 'ARRAY') {
+                next unless @$ctx[$var];
+                ($ctx, $value) = lookup($field, @$ctx[$var]);
+                last;
+            }
+        } elsif (ref $ctx eq 'HASH') {
+		    next unless exists $ctx->{$field};
             $value = $ctx->{$field};
             last;
+        } elsif (ref $ctx eq 'ARRAY') {
+            next unless @$ctx[$field];
+            $value = @$ctx[$field];
+            last;
         }
         elsif ($ctx && $blessed_or_not_ref && _can_run_field($ctx, $field)) {
             # We want to accept class names and objects, but not unblessed refs
@@ -440,6 +280,255 @@ sub _can_run_field {
 
 use namespace::clean;
 
+
+sub new {
+    my ($class, %args) = @_;
+    return bless({ %args }, $class);
+}
+
+our $template_path = '.';
+
+
+sub template_path { $Template::Mustache::template_path }
+
+our $template_extension = 'mustache';
+
+
+sub template_extension { $Template::Mustache::template_extension }
+
+
+sub template_namespace { '' }
+
+our $template_file;
+
+
+sub template_file {
+    my ($receiver) = @_;
+    return $Template::Mustache::template_file
+        if $Template::Mustache::template_file;
+
+    my $class = ref $receiver || $receiver;
+    $class =~ s/^@{[$receiver->template_namespace()]}:://;
+    my $ext  = $receiver->template_extension();
+    return File::Spec->catfile(split(/::/, "${class}.${ext}"));
+};
+
+
+sub template {
+    my ($receiver) = @_;
+    my $path = $receiver->template_path();
+    my $template_file = $receiver->template_file();
+    return read_file(File::Spec->catfile($path, $template_file));
+}
+
+
+sub partial {
+    my ($receiver, $name) = @_;
+    my $path = $receiver->template_path();
+    my $ext  = $receiver->template_extension();
+    return read_file(File::Spec->catfile($path, "${name}.${ext}"));
+}
+
+
+sub render {
+    my ($receiver, $tmpl, $data, $partials) = @_;
+    ($data, $tmpl) = ($tmpl, $data) if !(ref $data) && (ref $tmpl);
+
+    $tmpl       = $receiver->template() unless defined $tmpl;
+    $data     ||= $receiver;
+    $partials ||= sub {
+        unshift @_, $receiver;
+        goto &{$receiver->can('partial')};
+    };
+
+    my $part = $partials;
+    $part = sub { lookup(shift, $partials) } unless ref $partials eq 'CODE';
+
+    my $parsed = parse($tmpl);
+    return generate($parsed, $part, $data);
+}
+
+
+1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Template::Mustache - Drawing Mustaches on Perl for fun and profit
+
+=head1 VERSION
+
+version 0.5.6
+
+=head1 SYNOPSIS
+
+    use Template::Mustache;
+
+    print Template::Mustache->render(
+        "Hello {{planet}}", {planet => "World!"}), "\n";
+
+=head1 DESCRIPTION
+
+Template::Mustache is an implementation of the fabulous Mustache templating
+language for Perl 5.8 and later.
+
+See L<http://mustache.github.com>.
+
+=head2 Functions
+
+=over 4
+
+=item build_pattern($otag, $ctag)
+
+Constructs a new regular expression, to be used in the parsing of Mustache
+templates.
+
+=over 4
+
+=item $otag
+
+The tag opening delimiter.
+
+=item $ctag
+
+The tag closing delimiter.
+
+=back
+
+Returns a regular expression that will match tags with the specified
+delimiters.
+
+=item read_file($filename)
+
+Reads a file into a string, returning the empty string if the file does not
+exist.
+
+=over 4
+
+=item $filename
+
+The name of the file to read.
+
+=back
+
+Returns the contents of the given filename, or the empty string.
+
+=item parse($tmpl, [$delims, [$section, $start]])
+
+Can be called in one of three forms:
+
+=over 4
+
+=item parse($tmpl)
+
+Creates an AST from the given template.
+
+=over 4
+
+=item $tmpl
+
+The template to parse.
+
+=back
+
+An array reference to the AST represented by the given template.
+
+=item parse($tmpl, $delims)
+
+Creates an AST from the given template, with non-standard delimiters.
+
+=over 4
+
+=item $tmpl
+
+The template to parse.
+
+=item $delims
+
+An array reference to the delimiter pair with which to begin parsing.
+
+=back
+
+Returns an array reference to the AST represented by the given template.
+
+=item parse($tmpl, $delims, $section, $start)
+
+Parses out a section tag from the given template.
+
+=over 4
+
+=item $tmpl
+
+The template to parse.
+
+=item $delims
+
+An array reference to the delimiter pair with which to begin parsing.
+
+=item $section
+
+The name of the section we're parsing.
+
+=item $start
+
+The index of the first character of the section.
+
+=back
+
+Returns an array reference to the raw text of the section (first element),
+and the index of the character immediately following the close section tag
+(last element).
+
+=back
+
+=item generate($parse_tree, $partials, @context)
+
+Produces an expanded version of the template represented by the given parse
+tree.
+
+=over 4
+
+=item $parse_tree
+
+The AST of a Mustache template.
+
+=item $partials
+
+A subroutine that looks up partials by name.
+
+=item @context
+
+The context stack to perform key lookups against.
+
+=back
+
+Returns the fully rendered template as a string.
+
+=item lookup($field, @context)
+
+Performs a lookup of a C<$field> in a context stack.
+
+=over 4
+
+=item $field
+
+The field to look up.
+
+=item @context
+
+The context stack.
+
+=back
+
+Returns the context element and value for the given C<$field>.
+
+=back
+
 =head2 Methods
 
 =over 4
@@ -458,37 +547,18 @@ Initialization data.
 
 Returns A new C<Template::Mustache> instance.
 
-=cut
-
-sub new {
-    my ($class, %args) = @_;
-    return bless({ %args }, $class);
-}
-
-our $template_path = '.';
-
 =item template_path
 
 Filesystem path for template and partial lookups.
 
 Returns a string containing the template path (defaults to '.').
 
-=cut
-
-sub template_path { $Template::Mustache::template_path }
-
-our $template_extension = 'mustache';
-
 =item template_extension
 
 File extension for templates and partials.
 
 Returns the file extension as a string (defaults to 'mustache').
 
-=cut
-
-sub template_extension { $Template::Mustache::template_extension }
-
 =item template_namespace
 
 Package namespace to ignore during template lookups.
@@ -511,12 +581,6 @@ beginning of the package name.
 
 Returns the empty string.
 
-=cut
-
-sub template_namespace { '' }
-
-our $template_file;
-
 =item template_file
 
 The template filename to read.  The filename follows standard Perl module
@@ -553,34 +617,12 @@ Template files will be resolved against C<template_path>, not C<$PERL5LIB>.
 Returns The path to the template file, relative to C<template_path> as a
 string.  See L<template>.
 
-=cut
-
-sub template_file {
-    my ($receiver) = @_;
-    return $Template::Mustache::template_file
-        if $Template::Mustache::template_file;
-
-    my $class = ref $receiver || $receiver;
-    $class =~ s/^@{[$receiver->template_namespace()]}:://;
-    my $ext  = $receiver->template_extension();
-    return File::Spec->catfile(split(/::/, "${class}.${ext}"));
-};
-
 =item template
 
 Reads the template off disk.
 
 Returns the contents of the C<template_file> under C<template_path>.
 
-=cut
-
-sub template {
-    my ($receiver) = @_;
-    my $path = $receiver->template_path();
-    my $template_file = $receiver->template_file();
-    return read_file(File::Spec->catfile($path, $template_file));
-}
-
 =item partial($name)
 
 Reads a named partial off disk.
@@ -596,15 +638,6 @@ The name of the partial to lookup.
 Returns the contents of the partial (in C<template_path> of type
 C<template_extension>), or the empty string, if the partial does not exist.
 
-=cut
-
-sub partial {
-    my ($receiver, $name) = @_;
-    my $path = $receiver->template_path();
-    my $ext  = $receiver->template_extension();
-    return read_file(File::Spec->catfile($path, "${name}.${ext}"));
-}
-
 =item render
 
 Render a class or instances data, in each case returning the fully rendered
@@ -727,38 +760,31 @@ A hash containing partials.
 
 =back
 
-=cut
+=back
 
-sub render {
-    my ($receiver, $tmpl, $data, $partials) = @_;
-    ($data, $tmpl) = ($tmpl, $data) if !(ref $data) && (ref $tmpl);
+=head1 AUTHORS
 
-    $tmpl       = $receiver->template() unless defined $tmpl;
-    $data     ||= $receiver;
-    $partials ||= sub {
-        unshift @_, $receiver;
-        goto &{$receiver->can('partial')};
-    };
+=over 4
 
-    my $part = $partials;
-    $part = sub { lookup(shift, $partials) } unless ref $partials eq 'CODE';
+=item *
 
-    my $parsed = parse($tmpl);
-    return generate($parsed, $part, $data);
-}
+Pieter van de Bruggen <pvande@cpan.org>
+
+=item *
+
+Yanick Champoux <yanick@cpan.org>
+
+=item *
+
+Ricardo Signes <rjbs@cpan.org>
 
 =back
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-This program is free software; you can redistribute it and/or modify
-it only under the terms of Perl itself.
+This software is copyright (c) 2011 by Pieter van de Bruggen.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-Perl licensing terms for more details.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-1;
